@@ -1,7 +1,7 @@
 # repair.py - functions for repository repair for mercurial
 #
 # Copyright 2005, 2006 Chris Mason <mason@suse.com>
-# Copyright 2007 Matt Mackall
+# Copyright 2007 Olivia Mackall
 #
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
@@ -9,7 +9,6 @@
 from __future__ import absolute_import
 
 import errno
-import hashlib
 
 from .i18n import _
 from .node import (
@@ -24,49 +23,69 @@ from . import (
     exchange,
     obsolete,
     obsutil,
+    pathutil,
     phases,
     pycompat,
-    util,
+    requirements,
+    scmutil,
 )
 from .utils import (
+    hashutil,
     stringutil,
+    urlutil,
 )
 
-def backupbundle(repo, bases, heads, node, suffix, compress=True,
-                 obsolescence=True):
+
+def backupbundle(
+    repo, bases, heads, node, suffix, compress=True, obsolescence=True
+):
     """create a bundle with the specified revisions as a backup"""
 
-    backupdir = "strip-backup"
+    backupdir = b"strip-backup"
     vfs = repo.vfs
     if not vfs.isdir(backupdir):
         vfs.mkdir(backupdir)
 
     # Include a hash of all the nodes in the filename for uniqueness
-    allcommits = repo.set('%ln::%ln', bases, heads)
+    allcommits = repo.set(b'%ln::%ln', bases, heads)
     allhashes = sorted(c.hex() for c in allcommits)
-    totalhash = hashlib.sha1(''.join(allhashes)).digest()
-    name = "%s/%s-%s-%s.hg" % (backupdir, short(node),
-                               hex(totalhash[:4]), suffix)
+    totalhash = hashutil.sha1(b''.join(allhashes)).digest()
+    name = b"%s/%s-%s-%s.hg" % (
+        backupdir,
+        short(node),
+        hex(totalhash[:4]),
+        suffix,
+    )
 
     cgversion = changegroup.localversion(repo)
     comp = None
-    if cgversion != '01':
-        bundletype = "HG20"
+    if cgversion != b'01':
+        bundletype = b"HG20"
         if compress:
-            comp = 'BZ'
+            comp = b'BZ'
     elif compress:
-        bundletype = "HG10BZ"
+        bundletype = b"HG10BZ"
     else:
-        bundletype = "HG10UN"
+        bundletype = b"HG10UN"
 
-    outgoing = discovery.outgoing(repo, missingroots=bases, missingheads=heads)
+    outgoing = discovery.outgoing(repo, missingroots=bases, ancestorsof=heads)
     contentopts = {
-        'cg.version': cgversion,
-        'obsolescence': obsolescence,
-        'phases': True,
+        b'cg.version': cgversion,
+        b'obsolescence': obsolescence,
+        b'phases': True,
     }
-    return bundle2.writenewbundle(repo.ui, repo, 'strip', name, bundletype,
-                                  outgoing, contentopts, vfs, compression=comp)
+    return bundle2.writenewbundle(
+        repo.ui,
+        repo,
+        b'strip',
+        name,
+        bundletype,
+        outgoing,
+        contentopts,
+        vfs,
+        compression=comp,
+    )
+
 
 def _collectfiles(repo, striprev):
     """find out the filelogs affected by the strip"""
@@ -77,42 +96,46 @@ def _collectfiles(repo, striprev):
 
     return sorted(files)
 
+
 def _collectrevlog(revlog, striprev):
     _, brokenset = revlog.getstrippoint(striprev)
     return [revlog.linkrev(r) for r in brokenset]
 
-def _collectmanifest(repo, striprev):
-    return _collectrevlog(repo.manifestlog.getstorage(b''), striprev)
 
 def _collectbrokencsets(repo, files, striprev):
     """return the changesets which will be broken by the truncation"""
     s = set()
 
-    s.update(_collectmanifest(repo, striprev))
+    for revlog in manifestrevlogs(repo):
+        s.update(_collectrevlog(revlog, striprev))
     for fname in files:
         s.update(_collectrevlog(repo.file(fname), striprev))
 
     return s
 
-def strip(ui, repo, nodelist, backup=True, topic='backup'):
+
+def strip(ui, repo, nodelist, backup=True, topic=b'backup'):
     # This function requires the caller to lock the repo, but it operates
     # within a transaction of its own, and thus requires there to be no current
     # transaction when it is called.
     if repo.currenttransaction() is not None:
-        raise error.ProgrammingError('cannot strip from inside a transaction')
+        raise error.ProgrammingError(b'cannot strip from inside a transaction')
 
     # Simple way to maintain backwards compatibility for this
     # argument.
-    if backup in ['none', 'strip']:
+    if backup in [b'none', b'strip']:
         backup = False
 
     repo = repo.unfiltered()
     repo.destroying()
     vfs = repo.vfs
+    # load bookmark before changelog to avoid side effect from outdated
+    # changelog (see repo._refreshchangelog)
+    repo._bookmarks
     cl = repo.changelog
 
     # TODO handle undo of merge sets
-    if isinstance(nodelist, str):
+    if isinstance(nodelist, bytes):
         nodelist = [nodelist]
     striplist = [cl.rev(node) for node in nodelist]
     striprev = min(striplist)
@@ -147,11 +170,12 @@ def strip(ui, repo, nodelist, backup=True, topic='backup'):
     stripbases = [cl.node(r) for r in tostrip]
 
     stripobsidx = obsmarkers = ()
-    if repo.ui.configbool('devel', 'strip-obsmarkers'):
+    if repo.ui.configbool(b'devel', b'strip-obsmarkers'):
         obsmarkers = obsutil.exclusivemarkers(repo, stripbases)
     if obsmarkers:
-        stripobsidx = [i for i, m in enumerate(repo.obsstore)
-                       if m in obsmarkers]
+        stripobsidx = [
+            i for i, m in enumerate(repo.obsstore) if m in obsmarkers
+        ]
 
     newbmtarget, updatebm = _bookmarkmovements(repo, tostrip)
 
@@ -168,17 +192,25 @@ def strip(ui, repo, nodelist, backup=True, topic='backup'):
         # we are trying to strip.  This is harmless since the stripped markers
         # are already backed up and we did not touched the markers for the
         # saved changesets.
-        tmpbundlefile = backupbundle(repo, savebases, saveheads, node, 'temp',
-                                     compress=False, obsolescence=False)
+        tmpbundlefile = backupbundle(
+            repo,
+            savebases,
+            saveheads,
+            node,
+            b'temp',
+            compress=False,
+            obsolescence=False,
+        )
 
     with ui.uninterruptible():
         try:
-            with repo.transaction("strip") as tr:
+            with repo.transaction(b"strip") as tr:
                 # TODO this code violates the interface abstraction of the
                 # transaction and makes assumptions that file storage is
                 # using append-only files. We'll need some kind of storage
                 # API to handle stripping for us.
-                offset = len(tr._entries)
+                oldfiles = set(tr._offsetmap.keys())
+                oldfiles.update(tr._newfiles)
 
                 tr.startgroup()
                 cl.strip(striprev, tr)
@@ -188,9 +220,12 @@ def strip(ui, repo, nodelist, backup=True, topic='backup'):
                     repo.file(fn).strip(striprev, tr)
                 tr.endgroup()
 
-                for i in pycompat.xrange(offset, len(tr._entries)):
-                    file, troffset, ignore = tr._entries[i]
-                    with repo.svfs(file, 'a', checkambig=True) as fp:
+                entries = tr.readjournal()
+
+                for file, troffset in entries:
+                    if file in oldfiles:
+                        continue
+                    with repo.svfs(file, b'a', checkambig=True) as fp:
                         fp.truncate(troffset)
                     if troffset == 0:
                         repo.store.markremoved(file)
@@ -201,24 +236,25 @@ def strip(ui, repo, nodelist, backup=True, topic='backup'):
                 repo._phasecache.filterunknown(repo)
 
             if tmpbundlefile:
-                ui.note(_("adding branch\n"))
-                f = vfs.open(tmpbundlefile, "rb")
+                ui.note(_(b"adding branch\n"))
+                f = vfs.open(tmpbundlefile, b"rb")
                 gen = exchange.readbundle(ui, f, tmpbundlefile, vfs)
                 if not repo.ui.verbose:
                     # silence internal shuffling chatter
                     repo.ui.pushbuffer()
-                tmpbundleurl = 'bundle:' + vfs.join(tmpbundlefile)
-                txnname = 'strip'
+                tmpbundleurl = b'bundle:' + vfs.join(tmpbundlefile)
+                txnname = b'strip'
                 if not isinstance(gen, bundle2.unbundle20):
-                    txnname = "strip\n%s" % util.hidepassword(tmpbundleurl)
+                    txnname = b"strip\n%s" % urlutil.hidepassword(tmpbundleurl)
                 with repo.transaction(txnname) as tr:
-                    bundle2.applybundle(repo, gen, tr, source='strip',
-                                        url=tmpbundleurl)
+                    bundle2.applybundle(
+                        repo, gen, tr, source=b'strip', url=tmpbundleurl
+                    )
                 if not repo.ui.verbose:
                     repo.ui.popbuffer()
                 f.close()
 
-            with repo.transaction('repair') as tr:
+            with repo.transaction(b'repair') as tr:
                 bmchanges = [(m, repo[newbmtarget].node()) for m in updatebm]
                 repo._bookmarks.applychanges(repo, tr, bmchanges)
 
@@ -228,19 +264,32 @@ def strip(ui, repo, nodelist, backup=True, topic='backup'):
                     undovfs.unlink(undofile)
                 except OSError as e:
                     if e.errno != errno.ENOENT:
-                        ui.warn(_('error removing %s: %s\n') %
-                                (undovfs.join(undofile),
-                                 stringutil.forcebytestr(e)))
+                        ui.warn(
+                            _(b'error removing %s: %s\n')
+                            % (
+                                undovfs.join(undofile),
+                                stringutil.forcebytestr(e),
+                            )
+                        )
 
-        except: # re-raises
+        except:  # re-raises
             if backupfile:
-                ui.warn(_("strip failed, backup bundle stored in '%s'\n")
-                        % vfs.join(backupfile))
+                ui.warn(
+                    _(b"strip failed, backup bundle stored in '%s'\n")
+                    % vfs.join(backupfile)
+                )
             if tmpbundlefile:
-                ui.warn(_("strip failed, unrecovered changes stored in '%s'\n")
-                        % vfs.join(tmpbundlefile))
-                ui.warn(_("(fix the problem, then recover the changesets with "
-                          "\"hg unbundle '%s'\")\n") % vfs.join(tmpbundlefile))
+                ui.warn(
+                    _(b"strip failed, unrecovered changes stored in '%s'\n")
+                    % vfs.join(tmpbundlefile)
+                )
+                ui.warn(
+                    _(
+                        b"(fix the problem, then recover the changesets with "
+                        b"\"hg unbundle '%s'\")\n"
+                    )
+                    % vfs.join(tmpbundlefile)
+                )
             raise
         else:
             if tmpbundlefile:
@@ -252,6 +301,26 @@ def strip(ui, repo, nodelist, backup=True, topic='backup'):
     # extensions can use it
     return backupfile
 
+
+def softstrip(ui, repo, nodelist, backup=True, topic=b'backup'):
+    """perform a "soft" strip using the archived phase"""
+    tostrip = [c.node() for c in repo.set(b'sort(%ln::)', nodelist)]
+    if not tostrip:
+        return None
+
+    backupfile = None
+    if backup:
+        node = tostrip[0]
+        backupfile = _createstripbackup(repo, tostrip, node, topic)
+
+    newbmtarget, updatebm = _bookmarkmovements(repo, tostrip)
+    with repo.transaction(b'strip') as tr:
+        phases.retractboundary(repo, tr, phases.archived, tostrip)
+        bmchanges = [(m, repo[newbmtarget].node()) for m in updatebm]
+        repo._bookmarks.applychanges(repo, tr, bmchanges)
+    return backupfile
+
+
 def _bookmarkmovements(repo, tostrip):
     # compute necessary bookmark movement
     bm = repo._bookmarks
@@ -261,43 +330,50 @@ def _bookmarkmovements(repo, tostrip):
         if rev in tostrip:
             updatebm.append(m)
     newbmtarget = None
-    if updatebm: # don't compute anything is there is no bookmark to move anyway
+    # If we need to move bookmarks, compute bookmark
+    # targets. Otherwise we can skip doing this logic.
+    if updatebm:
         # For a set s, max(parents(s) - s) is the same as max(heads(::s - s)),
         # but is much faster
-        newbmtarget = repo.revs('max(parents(%ld) - (%ld))', tostrip, tostrip)
+        newbmtarget = repo.revs(b'max(parents(%ld) - (%ld))', tostrip, tostrip)
         if newbmtarget:
             newbmtarget = repo[newbmtarget.first()].node()
         else:
-            newbmtarget = '.'
+            newbmtarget = b'.'
     return newbmtarget, updatebm
 
+
 def _createstripbackup(repo, stripbases, node, topic):
     # backup the changeset we are about to strip
     vfs = repo.vfs
     cl = repo.changelog
     backupfile = backupbundle(repo, stripbases, cl.heads(), node, topic)
-    repo.ui.status(_("saved backup bundle to %s\n") %
-                   vfs.join(backupfile))
-    repo.ui.log("backupbundle", "saved backup bundle to %s\n",
-                vfs.join(backupfile))
+    repo.ui.status(_(b"saved backup bundle to %s\n") % vfs.join(backupfile))
+    repo.ui.log(
+        b"backupbundle", b"saved backup bundle to %s\n", vfs.join(backupfile)
+    )
     return backupfile
 
+
 def safestriproots(ui, repo, nodes):
     """return list of roots of nodes where descendants are covered by nodes"""
     torev = repo.unfiltered().changelog.rev
-    revs = set(torev(n) for n in nodes)
+    revs = {torev(n) for n in nodes}
     # tostrip = wanted - unsafe = wanted - ancestors(orphaned)
     # orphaned = affected - wanted
     # affected = descendants(roots(wanted))
     # wanted = revs
-    revset = '%ld - ( ::( (roots(%ld):: and not _phase(%s)) -%ld) )'
+    revset = b'%ld - ( ::( (roots(%ld):: and not _phase(%s)) -%ld) )'
     tostrip = set(repo.revs(revset, revs, revs, phases.internal, revs))
     notstrip = revs - tostrip
     if notstrip:
-        nodestr = ', '.join(sorted(short(repo[n].node()) for n in notstrip))
-        ui.warn(_('warning: orphaned descendants detected, '
-                  'not stripping %s\n') % nodestr)
-    return [c.node() for c in repo.set('roots(%ld)', tostrip)]
+        nodestr = b', '.join(sorted(short(repo[n].node()) for n in notstrip))
+        ui.warn(
+            _(b'warning: orphaned descendants detected, not stripping %s\n')
+            % nodestr
+        )
+    return [c.node() for c in repo.set(b'roots(%ld)', tostrip)]
+
 
 class stripcallback(object):
     """used as a transaction postclose callback"""
@@ -306,7 +382,7 @@ class stripcallback(object):
         self.ui = ui
         self.repo = repo
         self.backup = backup
-        self.topic = topic or 'backup'
+        self.topic = topic or b'backup'
         self.nodelist = []
 
     def addnodes(self, nodes):
@@ -317,6 +393,7 @@ class stripcallback(object):
         if roots:
             strip(self.ui, self.repo, roots, self.backup, self.topic)
 
+
 def delayedstrip(ui, repo, nodelist, topic=None, backup=True):
     """like strip, but works inside transaction and won't strip irreverent revs
 
@@ -332,27 +409,32 @@ def delayedstrip(ui, repo, nodelist, topic=None, backup=True):
         return strip(ui, repo, nodes, backup=backup, topic=topic)
     # transaction postclose callbacks are called in alphabet order.
     # use '\xff' as prefix so we are likely to be called last.
-    callback = tr.getpostclose('\xffstrip')
+    callback = tr.getpostclose(b'\xffstrip')
     if callback is None:
         callback = stripcallback(ui, repo, backup=backup, topic=topic)
-        tr.addpostclose('\xffstrip', callback)
+        tr.addpostclose(b'\xffstrip', callback)
     if topic:
         callback.topic = topic
     callback.addnodes(nodelist)
 
-def stripmanifest(repo, striprev, tr, files):
-    revlog = repo.manifestlog.getstorage(b'')
-    revlog.strip(striprev, tr)
-    striptrees(repo, tr, striprev, files)
 
-def striptrees(repo, tr, striprev, files):
-    if 'treemanifest' in repo.requirements: # safe but unnecessary
-                                            # otherwise
-        for unencoded, encoded, size in repo.store.datafiles():
-            if (unencoded.startswith('meta/') and
-                unencoded.endswith('00manifest.i')):
+def stripmanifest(repo, striprev, tr, files):
+    for revlog in manifestrevlogs(repo):
+        revlog.strip(striprev, tr)
+
+
+def manifestrevlogs(repo):
+    yield repo.manifestlog.getstorage(b'')
+    if scmutil.istreemanifest(repo):
+        # This logic is safe if treemanifest isn't enabled, but also
+        # pointless, so we skip it if treemanifest isn't enabled.
+        for t, unencoded, encoded, size in repo.store.datafiles():
+            if unencoded.startswith(b'meta/') and unencoded.endswith(
+                b'00manifest.i'
+            ):
                 dir = unencoded[5:-12]
-                repo.manifestlog.getstorage(dir).strip(striprev, tr)
+                yield repo.manifestlog.getstorage(dir)
+
 
 def rebuildfncache(ui, repo):
     """Rebuilds the fncache file from repo history.
@@ -361,23 +443,26 @@ def rebuildfncache(ui, repo):
     """
     repo = repo.unfiltered()
 
-    if 'fncache' not in repo.requirements:
-        ui.warn(_('(not rebuilding fncache because repository does not '
-                  'support fncache)\n'))
+    if requirements.FNCACHE_REQUIREMENT not in repo.requirements:
+        ui.warn(
+            _(
+                b'(not rebuilding fncache because repository does not '
+                b'support fncache)\n'
+            )
+        )
         return
 
     with repo.lock():
         fnc = repo.store.fncache
-        # Trigger load of fncache.
-        if 'irrelevant' in fnc:
-            pass
+        fnc.ensureloaded(warn=ui.warn)
 
         oldentries = set(fnc.entries)
         newentries = set()
         seenfiles = set()
 
-        progress = ui.makeprogress(_('rebuilding'), unit=_('changesets'),
-                                   total=len(repo))
+        progress = ui.makeprogress(
+            _(b'rebuilding'), unit=_(b'changesets'), total=len(repo)
+        )
         for rev in repo:
             progress.update(rev)
 
@@ -388,8 +473,8 @@ def rebuildfncache(ui, repo):
                     continue
                 seenfiles.add(f)
 
-                i = 'data/%s.i' % f
-                d = 'data/%s.d' % f
+                i = b'data/%s.i' % f
+                d = b'data/%s.d' % f
 
                 if repo.store._exists(i):
                     newentries.add(i)
@@ -398,10 +483,12 @@ def rebuildfncache(ui, repo):
 
         progress.complete()
 
-        if 'treemanifest' in repo.requirements: # safe but unnecessary otherwise
-            for dir in util.dirs(seenfiles):
-                i = 'meta/%s/00manifest.i' % dir
-                d = 'meta/%s/00manifest.d' % dir
+        if requirements.TREEMANIFEST_REQUIREMENT in repo.requirements:
+            # This logic is safe if treemanifest isn't enabled, but also
+            # pointless, so we skip it if treemanifest isn't enabled.
+            for dir in pathutil.dirs(seenfiles):
+                i = b'meta/%s/00manifest.i' % dir
+                d = b'meta/%s/00manifest.d' % dir
 
                 if repo.store._exists(i):
                     newentries.add(i)
@@ -411,20 +498,23 @@ def rebuildfncache(ui, repo):
         addcount = len(newentries - oldentries)
         removecount = len(oldentries - newentries)
         for p in sorted(oldentries - newentries):
-            ui.write(_('removing %s\n') % p)
+            ui.write(_(b'removing %s\n') % p)
         for p in sorted(newentries - oldentries):
-            ui.write(_('adding %s\n') % p)
+            ui.write(_(b'adding %s\n') % p)
 
         if addcount or removecount:
-            ui.write(_('%d items added, %d removed from fncache\n') %
-                     (addcount, removecount))
+            ui.write(
+                _(b'%d items added, %d removed from fncache\n')
+                % (addcount, removecount)
+            )
             fnc.entries = newentries
             fnc._dirty = True
 
-            with repo.transaction('fncache') as tr:
+            with repo.transaction(b'fncache') as tr:
                 fnc.write(tr)
         else:
-            ui.write(_('fncache already up to date\n'))
+            ui.write(_(b'fncache already up to date\n'))
+
 
 def deleteobsmarkers(obsstore, indices):
     """Delete some obsmarkers from obsstore and return how many were deleted
@@ -449,7 +539,7 @@ def deleteobsmarkers(obsstore, indices):
             continue
         left.append(m)
 
-    newobsstorefile = obsstore.svfs('obsstore', 'w', atomictemp=True)
+    newobsstorefile = obsstore.svfs(b'obsstore', b'w', atomictemp=True)
     for bytes in obsolete.encodemarkers(left, True, obsstore._version):
         newobsstorefile.write(bytes)
     newobsstorefile.close()
