@@ -1,6 +1,6 @@
 # templatekw.py - common changeset template keywords
 #
-# Copyright 2005-2009 Matt Mackall <mpm@selenic.com>
+# Copyright 2005-2009 Olivia Mackall <olivia@selenic.com>
 #
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
@@ -422,7 +422,7 @@ def getgraphnodecurrent(repo, ctx, cache):
             from . import mergestate as mergestatemod
 
             mergestate = mergestatemod.mergestate.read(repo)
-            if mergestate.active():
+            if mergestate.unresolvedcount():
                 merge_nodes = (mergestate.local, mergestate.other)
             else:
                 merge_nodes = ()
@@ -634,6 +634,19 @@ def showobsolete(context, mapping):
     return b''
 
 
+@templatekeyword(b'onelinesummary', requires={b'ui', b'ctx'})
+def showonelinesummary(context, mapping):
+    """String. A one-line summary for the ctx (not including trailing newline).
+    The default template be overridden in command-templates.oneline-summary."""
+    # Avoid cycle:
+    # mercurial.cmdutil -> mercurial.templatekw -> mercurial.cmdutil
+    from . import cmdutil
+
+    ui = context.resource(mapping, b'ui')
+    ctx = context.resource(mapping, b'ctx')
+    return cmdutil.format_changeset_summary(ui, ctx)
+
+
 @templatekeyword(b'path', requires={b'fctx'})
 def showpath(context, mapping):
     """String. Repository-absolute path of the current file. (EXPERIMENTAL)"""
@@ -712,21 +725,20 @@ def showsuccessorssets(context, mapping):
     while also diverged into ctx3. (EXPERIMENTAL)"""
     repo = context.resource(mapping, b'repo')
     ctx = context.resource(mapping, b'ctx')
-    if not ctx.obsolete():
-        return b''
-
-    ssets = obsutil.successorssets(repo, ctx.node(), closest=True)
-    ssets = [[hex(n) for n in ss] for ss in ssets]
-
     data = []
-    for ss in ssets:
-        h = _hybrid(
-            None,
-            ss,
-            lambda x: {b'ctx': repo[x]},
-            lambda x: scmutil.formatchangeid(repo[x]),
-        )
-        data.append(h)
+
+    if ctx.obsolete():
+        ssets = obsutil.successorssets(repo, ctx.node(), closest=True)
+        ssets = [[hex(n) for n in ss] for ss in ssets]
+
+        for ss in ssets:
+            h = _hybrid(
+                None,
+                ss,
+                lambda x: {b'ctx': repo[x]},
+                lambda x: scmutil.formatchangeid(repo[x]),
+            )
+            data.append(h)
 
     # Format the successorssets
     def render(d):
@@ -982,8 +994,7 @@ def showwhyunstable(context, mapping):
 
 
 def loadkeyword(ui, extname, registrarobj):
-    """Load template keyword from specified registrarobj
-    """
+    """Load template keyword from specified registrarobj"""
     for name, func in pycompat.iteritems(registrarobj._table):
         keywords[name] = func
 
