@@ -1,6 +1,6 @@
 # common.py - common code for the convert extension
 #
-#  Copyright 2005-2009 Matt Mackall <mpm@selenic.com> and others
+#  Copyright 2005-2009 Olivia Mackall <olivia@selenic.com> and others
 #
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
@@ -84,10 +84,18 @@ def shlexer(data=None, filepath=None, wordchars=None, whitespace=None):
     return l
 
 
+if pycompat.ispy3:
+    base64_encodebytes = base64.encodebytes
+    base64_decodebytes = base64.decodebytes
+else:
+    base64_encodebytes = base64.encodestring
+    base64_decodebytes = base64.decodestring
+
+
 def encodeargs(args):
     def encodearg(s):
-        lines = base64.encodestring(s)
-        lines = [l.splitlines()[0] for l in lines]
+        lines = base64_encodebytes(s)
+        lines = [l.splitlines()[0] for l in pycompat.iterbytestr(lines)]
         return b''.join(lines)
 
     s = pickle.dumps(args)
@@ -95,7 +103,7 @@ def encodeargs(args):
 
 
 def decodeargs(s):
-    s = base64.decodestring(s)
+    s = base64_decodebytes(s)
     return pickle.loads(s)
 
 
@@ -164,8 +172,8 @@ class converter_source(object):
         self.encoding = b'utf-8'
 
     def checkhexformat(self, revstr, mapname=b'splicemap'):
-        """ fails if revstr is not a 40 byte hex. mercurial and git both uses
-            such format for their revision numbering
+        """fails if revstr is not a 40 byte hex. mercurial and git both uses
+        such format for their revision numbering
         """
         if not re.match(br'[0-9a-fA-F]{40,40}$', revstr):
             raise error.Abort(
@@ -275,8 +283,7 @@ class converter_source(object):
         return False
 
     def hasnativeclose(self):
-        """Return true if this source has ability to close branch.
-        """
+        """Return true if this source has ability to close branch."""
         return False
 
     def lookuprev(self, rev):
@@ -295,8 +302,8 @@ class converter_source(object):
 
     def checkrevformat(self, revstr, mapname=b'splicemap'):
         """revstr is a string that describes a revision in the given
-           source control system.  Return true if revstr has correct
-           format.
+        source control system.  Return true if revstr has correct
+        format.
         """
         return True
 
