@@ -73,6 +73,8 @@ beneficial to performance but isn't enabled.
 
     [fsmonitor]
     warn_update_file_count = (integer)
+    # or when mercurial is built with rust support
+    warn_update_file_count_rust = (integer)
 
 If ``warn_when_unused`` is set and fsmonitor isn't enabled, a warning will
 be printed during working directory updates if this many files will be
@@ -152,25 +154,40 @@ configtable = {}
 configitem = registrar.configitem(configtable)
 
 configitem(
-    b'fsmonitor', b'mode', default=b'on',
+    b'fsmonitor',
+    b'mode',
+    default=b'on',
 )
 configitem(
-    b'fsmonitor', b'walk_on_invalidate', default=False,
+    b'fsmonitor',
+    b'walk_on_invalidate',
+    default=False,
 )
 configitem(
-    b'fsmonitor', b'timeout', default=b'2',
+    b'fsmonitor',
+    b'timeout',
+    default=b'2',
 )
 configitem(
-    b'fsmonitor', b'blacklistusers', default=list,
+    b'fsmonitor',
+    b'blacklistusers',
+    default=list,
 )
 configitem(
-    b'fsmonitor', b'watchman_exe', default=b'watchman',
+    b'fsmonitor',
+    b'watchman_exe',
+    default=b'watchman',
 )
 configitem(
-    b'fsmonitor', b'verbose', default=True, experimental=True,
+    b'fsmonitor',
+    b'verbose',
+    default=True,
+    experimental=True,
 )
 configitem(
-    b'experimental', b'fsmonitor.transaction_notify', default=False,
+    b'experimental',
+    b'fsmonitor.transaction_notify',
+    default=False,
 )
 
 # This extension is incompatible with the following blacklisted extensions
@@ -269,11 +286,11 @@ def _watchmantofsencoding(path):
 
 
 def overridewalk(orig, self, match, subrepos, unknown, ignored, full=True):
-    '''Replacement for dirstate.walk, hooking into Watchman.
+    """Replacement for dirstate.walk, hooking into Watchman.
 
     Whenever full is False, ignored is False, and the Watchman client is
     available, use Watchman combined with saved state to possibly return only a
-    subset of files.'''
+    subset of files."""
 
     def bail(reason):
         self._ui.debug(b'fsmonitor: fallback to core status, %s\n' % reason)
@@ -397,7 +414,7 @@ def overridewalk(orig, self, match, subrepos, unknown, ignored, full=True):
     # for file paths which require normalization and we encounter a case
     # collision, we store our own foldmap
     if normalize:
-        foldmap = dict((normcase(k), k) for k in results)
+        foldmap = {normcase(k): k for k in results}
 
     switch_slashes = pycompat.ossep == b'\\'
     # The order of the results is, strictly speaking, undefined.
@@ -459,22 +476,16 @@ def overridewalk(orig, self, match, subrepos, unknown, ignored, full=True):
     if normalize:
         # any notable files that have changed case will already be handled
         # above, so just check membership in the foldmap
-        notefiles = set(
-            (
-                normalize(f, True, True)
-                for f in notefiles
-                if normcase(f) not in foldmap
-            )
-        )
-    visit = set(
-        (
-            f
+        notefiles = {
+            normalize(f, True, True)
             for f in notefiles
-            if (
-                f not in results and matchfn(f) and (f in dmap or not ignore(f))
-            )
-        )
-    )
+            if normcase(f) not in foldmap
+        }
+    visit = {
+        f
+        for f in notefiles
+        if (f not in results and matchfn(f) and (f in dmap or not ignore(f)))
+    }
 
     if not fresh_instance:
         if matchalways:
@@ -673,7 +684,7 @@ def overridestatus(
 
 class poststatus(object):
     def __init__(self, startclock):
-        self._startclock = startclock
+        self._startclock = pycompat.sysbytes(startclock)
 
     def __call__(self, wctx, status):
         clock = wctx.repo()._fsmonitorstate.getlastclock() or self._startclock
@@ -731,12 +742,12 @@ def extsetup(ui):
         # An assist for avoiding the dangling-symlink fsevents bug
         extensions.wrapfunction(os, b'symlink', wrapsymlink)
 
-    extensions.wrapfunction(merge, b'update', wrapupdate)
+    extensions.wrapfunction(merge, b'_update', wrapupdate)
 
 
 def wrapsymlink(orig, source, link_name):
-    ''' if we create a dangling symlink, also touch the parent dir
-    to encourage fsevents notifications to work more correctly '''
+    """if we create a dangling symlink, also touch the parent dir
+    to encourage fsevents notifications to work more correctly"""
     try:
         return orig(source, link_name)
     finally:
@@ -747,13 +758,13 @@ def wrapsymlink(orig, source, link_name):
 
 
 class state_update(object):
-    ''' This context manager is responsible for dispatching the state-enter
-        and state-leave signals to the watchman service. The enter and leave
-        methods can be invoked manually (for scenarios where context manager
-        semantics are not possible). If parameters oldnode and newnode are None,
-        they will be populated based on current working copy in enter and
-        leave, respectively. Similarly, if the distance is none, it will be
-        calculated based on the oldnode and newnode in the leave method.'''
+    """This context manager is responsible for dispatching the state-enter
+    and state-leave signals to the watchman service. The enter and leave
+    methods can be invoked manually (for scenarios where context manager
+    semantics are not possible). If parameters oldnode and newnode are None,
+    they will be populated based on current working copy in enter and
+    leave, respectively. Similarly, if the distance is none, it will be
+    calculated based on the oldnode and newnode in the leave method."""
 
     def __init__(
         self,
