@@ -1,6 +1,6 @@
 # ancestor.py - generic DAG ancestor algorithm for mercurial
 #
-# Copyright 2006 Matt Mackall <mpm@selenic.com>
+# Copyright 2006 Olivia Mackall <olivia@selenic.com>
 #
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
@@ -138,7 +138,7 @@ def ancestors(pfunc, *orignodes):
         k = 0
         for i in interesting:
             k |= i
-        return set(n for (i, n) in mapping if k & i)
+        return {n for (i, n) in mapping if k & i}
 
     gca = commonancestorsheads(pfunc, *orignodes)
 
@@ -148,11 +148,11 @@ def ancestors(pfunc, *orignodes):
 
 
 class incrementalmissingancestors(object):
-    '''persistent state used to calculate missing ancestors incrementally
+    """persistent state used to calculate missing ancestors incrementally
 
     Although similar in spirit to lazyancestors below, this is a separate class
     because trying to support contains and missingancestors operations with the
-    same internal data structures adds needless complexity.'''
+    same internal data structures adds needless complexity."""
 
     def __init__(self, pfunc, bases):
         self.bases = set(bases)
@@ -198,12 +198,12 @@ class incrementalmissingancestors(object):
                 break
 
     def missingancestors(self, revs):
-        '''return all the ancestors of revs that are not ancestors of self.bases
+        """return all the ancestors of revs that are not ancestors of self.bases
 
         This may include elements from revs.
 
         Equivalent to the revset (::revs - ::self.bases). Revs are returned in
-        revision number order, which is a topological order.'''
+        revision number order, which is a topological order."""
         revsvisit = set(revs)
         basesvisit = self.bases
         pfunc = self.pfunc
@@ -393,39 +393,3 @@ class lazyancestors(object):
             # free up memory.
             self._containsiter = None
             return False
-
-
-class rustlazyancestors(object):
-    def __init__(self, index, revs, stoprev=0, inclusive=False):
-        self._index = index
-        self._stoprev = stoprev
-        self._inclusive = inclusive
-        # no need to prefilter out init revs that are smaller than stoprev,
-        # it's done by rustlazyancestors constructor.
-        # we need to convert to a list, because our ruslazyancestors
-        # constructor (from C code) doesn't understand anything else yet
-        self._initrevs = initrevs = list(revs)
-
-        self._containsiter = parsers.rustlazyancestors(
-            index, initrevs, stoprev, inclusive
-        )
-
-    def __nonzero__(self):
-        """False if the set is empty, True otherwise.
-
-        It's better to duplicate this essentially trivial method than
-        to subclass lazyancestors
-        """
-        try:
-            next(iter(self))
-            return True
-        except StopIteration:
-            return False
-
-    def __iter__(self):
-        return parsers.rustlazyancestors(
-            self._index, self._initrevs, self._stoprev, self._inclusive
-        )
-
-    def __contains__(self, target):
-        return target in self._containsiter
