@@ -1,6 +1,6 @@
 # filelog.py - file history class for mercurial
 #
-# Copyright 2005-2007 Matt Mackall <mpm@selenic.com>
+# Copyright 2005-2007 Olivia Mackall <olivia@selenic.com>
 #
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
@@ -14,23 +14,26 @@ from .node import (
 )
 from . import (
     error,
-    repository,
     revlog,
 )
-from .utils import (
-    interfaceutil,
-    storageutil,
+from .interfaces import (
+    repository,
+    util as interfaceutil,
 )
+from .utils import storageutil
+
 
 @interfaceutil.implementer(repository.ifilestorage)
 class filelog(object):
     def __init__(self, opener, path):
-        self._revlog = revlog.revlog(opener,
-                                     '/'.join(('data', path + '.i')),
-                                     censorable=True)
+        self._revlog = revlog.revlog(
+            opener, b'/'.join((b'data', path + b'.i')), censorable=True
+        )
         # Full name of the user visible file, relative to the repository root.
         # Used by LFS.
         self._revlog.filename = path
+        self._revlog.revlog_kind = b'filelog'
+        self.nullid = self._revlog.nullid
 
     def __len__(self):
         return len(self._revlog)
@@ -64,8 +67,9 @@ class filelog(object):
         return self._revlog.node(rev)
 
     def lookup(self, node):
-        return storageutil.fileidlookup(self._revlog, node,
-                                        self._revlog.indexfile)
+        return storageutil.fileidlookup(
+            self._revlog, node, self._revlog.indexfile
+        )
 
     def linkrev(self, rev):
         return self._revlog.linkrev(rev)
@@ -90,29 +94,73 @@ class filelog(object):
     def revision(self, node, _df=None, raw=False):
         return self._revlog.revision(node, _df=_df, raw=raw)
 
-    def emitrevisions(self, nodes, nodesorder=None,
-                      revisiondata=False, assumehaveparentrevisions=False,
-                      deltamode=repository.CG_DELTAMODE_STD):
+    def rawdata(self, node, _df=None):
+        return self._revlog.rawdata(node, _df=_df)
+
+    def emitrevisions(
+        self,
+        nodes,
+        nodesorder=None,
+        revisiondata=False,
+        assumehaveparentrevisions=False,
+        deltamode=repository.CG_DELTAMODE_STD,
+        sidedata_helpers=None,
+    ):
         return self._revlog.emitrevisions(
-            nodes, nodesorder=nodesorder, revisiondata=revisiondata,
+            nodes,
+            nodesorder=nodesorder,
+            revisiondata=revisiondata,
             assumehaveparentrevisions=assumehaveparentrevisions,
-            deltamode=deltamode)
+            deltamode=deltamode,
+            sidedata_helpers=sidedata_helpers,
+        )
 
-    def addrevision(self, revisiondata, transaction, linkrev, p1, p2,
-                    node=None, flags=revlog.REVIDX_DEFAULT_FLAGS,
-                    cachedelta=None):
-        return self._revlog.addrevision(revisiondata, transaction, linkrev,
-                                    p1, p2, node=node, flags=flags,
-                                    cachedelta=cachedelta)
+    def addrevision(
+        self,
+        revisiondata,
+        transaction,
+        linkrev,
+        p1,
+        p2,
+        node=None,
+        flags=revlog.REVIDX_DEFAULT_FLAGS,
+        cachedelta=None,
+    ):
+        return self._revlog.addrevision(
+            revisiondata,
+            transaction,
+            linkrev,
+            p1,
+            p2,
+            node=node,
+            flags=flags,
+            cachedelta=cachedelta,
+        )
 
-    def addgroup(self, deltas, linkmapper, transaction, addrevisioncb=None,
-                 maybemissingparents=False):
+    def addgroup(
+        self,
+        deltas,
+        linkmapper,
+        transaction,
+        addrevisioncb=None,
+        duplicaterevisioncb=None,
+        maybemissingparents=False,
+    ):
         if maybemissingparents:
-            raise error.Abort(_('revlog storage does not support missing '
-                                'parents write mode'))
+            raise error.Abort(
+                _(
+                    b'revlog storage does not support missing '
+                    b'parents write mode'
+                )
+            )
 
-        return self._revlog.addgroup(deltas, linkmapper, transaction,
-                                     addrevisioncb=addrevisioncb)
+        return self._revlog.addgroup(
+            deltas,
+            linkmapper,
+            transaction,
+            addrevisioncb=addrevisioncb,
+            duplicaterevisioncb=duplicaterevisioncb,
+        )
 
     def getstrippoint(self, minlink):
         return self._revlog.getstrippoint(minlink)
@@ -130,9 +178,10 @@ class filelog(object):
         return storageutil.filtermetadata(self.revision(node))
 
     def add(self, text, meta, transaction, link, p1=None, p2=None):
-        if meta or text.startswith('\1\n'):
+        if meta or text.startswith(b'\1\n'):
             text = storageutil.packmeta(meta, text)
-        return self.addrevision(text, transaction, link, p1, p2)
+        rev = self.addrevision(text, transaction, link, p1, p2)
+        return self.node(rev)
 
     def renamed(self, node):
         return storageutil.filerevisioncopied(self, node)
@@ -160,13 +209,21 @@ class filelog(object):
     def verifyintegrity(self, state):
         return self._revlog.verifyintegrity(state)
 
-    def storageinfo(self, exclusivefiles=False, sharedfiles=False,
-                    revisionscount=False, trackedsize=False,
-                    storedsize=False):
+    def storageinfo(
+        self,
+        exclusivefiles=False,
+        sharedfiles=False,
+        revisionscount=False,
+        trackedsize=False,
+        storedsize=False,
+    ):
         return self._revlog.storageinfo(
-            exclusivefiles=exclusivefiles, sharedfiles=sharedfiles,
-            revisionscount=revisionscount, trackedsize=trackedsize,
-            storedsize=storedsize)
+            exclusivefiles=exclusivefiles,
+            sharedfiles=sharedfiles,
+            revisionscount=revisionscount,
+            trackedsize=trackedsize,
+            storedsize=storedsize,
+        )
 
     # TODO these aren't part of the interface and aren't internal methods.
     # Callers should be fixed to not use them.
@@ -183,10 +240,11 @@ class filelog(object):
     # Used by repo upgrade.
     def clone(self, tr, destrevlog, **kwargs):
         if not isinstance(destrevlog, filelog):
-            raise error.ProgrammingError('expected filelog to clone()')
+            raise error.ProgrammingError(b'expected filelog to clone()')
 
         return self._revlog.clone(tr, destrevlog._revlog, **kwargs)
 
+
 class narrowfilelog(filelog):
     """Filelog variation to be used with narrow stores."""
 
@@ -226,14 +284,12 @@ class narrowfilelog(filelog):
             return super(narrowfilelog, self).size(rev)
 
     def cmp(self, node, text):
-        different = super(narrowfilelog, self).cmp(node, text)
+        # We don't call `super` because narrow parents can be buggy in case of a
+        # ambiguous dirstate. Always take the slow path until there is a better
+        # fix, see issue6150.
 
-        # Because renamed() may lie, we may get false positives for
-        # different content. Check for this by comparing against the original
-        # renamed() implementation.
-        if different:
-            if super(narrowfilelog, self).renamed(node):
-                t2 = self.read(node)
-                return t2 != text
+        # Censored files compare against the empty file.
+        if self.iscensored(self.rev(node)):
+            return text != b''
 
-        return different
+        return self.read(node) != text
