@@ -1,6 +1,6 @@
 # ui.py - user interface bits for mercurial
 #
-# Copyright 2005-2007 Matt Mackall <mpm@selenic.com>
+# Copyright 2005-2007 Olivia Mackall <olivia@selenic.com>
 #
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
@@ -9,6 +9,7 @@ from __future__ import absolute_import
 
 import collections
 import contextlib
+import datetime
 import errno
 import getpass
 import inspect
@@ -25,7 +26,6 @@ from .node import hex
 from .pycompat import (
     getattr,
     open,
-    setattr,
 )
 
 from . import (
@@ -47,6 +47,7 @@ from .utils import (
     procutil,
     resourceutil,
     stringutil,
+    urlutil,
 )
 
 urlreq = util.urlreq
@@ -242,6 +243,7 @@ class ui(object):
         self._terminfoparams = {}
         self._styles = {}
         self._uninterruptible = False
+        self.showtimestamp = False
 
         if src:
             self._fout = src._fout
@@ -300,6 +302,11 @@ class ui(object):
                 if k in self.environ:
                     self._exportableenviron[k] = self.environ[k]
 
+    def _new_source(self):
+        self._ocfg.new_source()
+        self._tcfg.new_source()
+        self._ucfg.new_source()
+
     @classmethod
     def load(cls):
         """Create a ui and load global and user configs"""
@@ -311,6 +318,7 @@ class ui(object):
             elif t == b'resource':
                 u.read_resource_config(f, trust=True)
             elif t == b'items':
+                u._new_source()
                 sections = set()
                 for section, name, value, source in f:
                     # do not set u._ocfg
@@ -323,6 +331,7 @@ class ui(object):
             else:
                 raise error.ProgrammingError(b'unknown rctype: %s' % t)
         u._maybetweakdefaults()
+        u._new_source()  # anything after that is a different level
         return u
 
     def _maybetweakdefaults(self):
@@ -462,10 +471,12 @@ class ui(object):
 
             try:
                 cfg.read(filename, fp, sections=sections, remap=remap)
-            except error.ParseError as inst:
+            except error.ConfigError as inst:
                 if trusted:
                     raise
-                self.warn(_(b'ignored: %s\n') % stringutil.forcebytestr(inst))
+                self.warn(
+                    _(b'ignored %s: %s\n') % (inst.location, inst.message)
+                )
 
         self._applyconfig(cfg, trusted, root)
 
@@ -505,6 +516,8 @@ class ui(object):
                 del cfg[b'defaults'][k]
             for k, v in cfg.items(b'commands'):
                 del cfg[b'commands'][k]
+            for k, v in cfg.items(b'command-templates'):
+                del cfg[b'command-templates'][k]
         # Don't remove aliases from the configuration if in the exceptionlist
         if self.plain(b'alias'):
             for k, v in cfg.items(b'alias'):
@@ -546,9 +559,9 @@ class ui(object):
                         )
                         p = p.replace(b'%%', b'%')
                     p = util.expandpath(p)
-                    if not util.hasscheme(p) and not os.path.isabs(p):
+                    if not urlutil.hasscheme(p) and not os.path.isabs(p):
                         p = os.path.normpath(os.path.join(root, p))
-                    c.set(b"paths", n, p)
+                    c.alter(b"paths", n, p)
 
         if section in (None, b'ui'):
             # update ui options
@@ -561,6 +574,7 @@ class ui(object):
             self._reportuntrusted = self.debugflag or self.configbool(
                 b"ui", b"report_untrusted"
             )
+            self.showtimestamp = self.configbool(b'ui', b'timestamp-output')
             self.tracebackflag = self.configbool(b'ui', b'traceback')
             self.logblockedtimes = self.configbool(b'ui', b'logblockedtimes')
 
@@ -648,11 +662,18 @@ class ui(object):
             msg %= (section, name, pycompat.bytestr(default))
             self.develwarn(msg, 2, b'warn-config-default')
 
+        candidates = []
+        config = self._data(untrusted)
         for s, n in alternates:
-            candidate = self._data(untrusted).get(s, n, None)
+            candidate = config.get(s, n, None)
             if candidate is not None:
-                value = candidate
-                break
+                candidates.append((s, n, candidate))
+        if candidates:
+
+            def level(x):
+                return config.level(x[0], x[1])
+
+            value = max(candidates, key=level)[2]
 
         if self.debugflag and not untrusted and self._reportuntrusted:
             for s, n in alternates:
@@ -664,6 +685,18 @@ class ui(object):
                     )
         return value
 
+    def config_default(self, section, name):
+        """return the default value for a config option
+
+        The default is returned "raw", for example if it is a callable, the
+        callable was not called.
+        """
+        item = self._knownconfig.get(section, {}).get(name)
+
+        if item is None:
+            raise KeyError((section, name))
+        return item.default
+
     def configsuboptions(self, section, name, default=_unset, untrusted=False):
         """Get a config option and all sub-options.
 
@@ -916,7 +949,7 @@ class ui(object):
                 yield section, name, value
 
     def plain(self, feature=None):
-        '''is plain mode active?
+        """is plain mode active?
 
         Plain mode means that all configuration variables which affect
         the behavior and output of Mercurial should be
@@ -930,7 +963,7 @@ class ui(object):
         - False if HGPLAIN is not set, or feature is in HGPLAINEXCEPT
         - False if feature is disabled by default and not included in HGPLAIN
         - True otherwise
-        '''
+        """
         if (
             b'HGPLAIN' not in encoding.environ
             and b'HGPLAINEXCEPT' not in encoding.environ
@@ -997,8 +1030,10 @@ class ui(object):
 
     def expandpath(self, loc, default=None):
         """Return repository location relative to cwd or from [paths]"""
+        msg = b'ui.expandpath is deprecated, use `get_*` functions from urlutil'
+        self.deprecwarn(msg, b'6.0')
         try:
-            p = self.paths.getpath(loc)
+            p = self.getpath(loc)
             if p:
                 return p.rawloc
         except error.RepoError:
@@ -1006,7 +1041,7 @@ class ui(object):
 
         if default:
             try:
-                p = self.paths.getpath(default)
+                p = self.getpath(default)
                 if p:
                     return p.rawloc
             except error.RepoError:
@@ -1016,7 +1051,14 @@ class ui(object):
 
     @util.propertycache
     def paths(self):
-        return paths(self)
+        return urlutil.paths(self)
+
+    def getpath(self, *args, **kwargs):
+        """see paths.getpath for details
+
+        This method exist as `getpath` need a ui for potential warning message.
+        """
+        return self.paths.getpath(self, *args, **kwargs)
 
     @property
     def fout(self):
@@ -1103,7 +1145,7 @@ class ui(object):
         return self._colormode != b'win32'
 
     def write(self, *args, **opts):
-        '''write args to output
+        """write args to output
 
         By default, this method simply writes to the buffer or stdout.
         Color mode can be set on the UI class to have the output decorated
@@ -1124,7 +1166,7 @@ class ui(object):
         When labeling output for a specific command, a label of
         "cmdname.type" is recommended. For example, status issues
         a label of "status.modified" for modified files.
-        '''
+        """
         dest = self._fout
 
         # inlined _write() for speed
@@ -1200,7 +1242,7 @@ class ui(object):
                 dest.write(msg)
             # stderr may be buffered under win32 when redirected to files,
             # including stdout.
-            if dest is self._ferr and not getattr(self._ferr, 'closed', False):
+            if dest is self._ferr and not getattr(dest, 'closed', False):
                 dest.flush()
         except IOError as err:
             if dest is self._ferr and err.errno in (
@@ -1217,7 +1259,21 @@ class ui(object):
             ) * 1000
 
     def _writemsg(self, dest, *args, **opts):
+        timestamp = self.showtimestamp and opts.get('type') in {
+            b'debug',
+            b'error',
+            b'note',
+            b'status',
+            b'warning',
+        }
+        if timestamp:
+            args = (
+                b'[%s] '
+                % pycompat.bytestr(datetime.datetime.now().isoformat()),
+            ) + args
         _writemsgwith(self._write, dest, *args, **opts)
+        if timestamp:
+            dest.flush()
 
     def _writemsgnobuf(self, dest, *args, **opts):
         _writemsgwith(self._writenobuf, dest, *args, **opts)
@@ -1430,9 +1486,9 @@ class ui(object):
         return _reqexithandlers
 
     def atexit(self, func, *args, **kwargs):
-        '''register a function to run after dispatching a request
+        """register a function to run after dispatching a request
 
-        Handlers do not stay registered across request boundaries.'''
+        Handlers do not stay registered across request boundaries."""
         self._exithandlers.append((func, args, kwargs))
         return func
 
@@ -1461,8 +1517,14 @@ class ui(object):
         alldefaults = frozenset([b"text", b"curses"])
 
         featureinterfaces = {
-            b"chunkselector": [b"text", b"curses",],
-            b"histedit": [b"text", b"curses",],
+            b"chunkselector": [
+                b"text",
+                b"curses",
+            ],
+            b"histedit": [
+                b"text",
+                b"curses",
+            ],
         }
 
         # Feature-specific interface
@@ -1509,7 +1571,7 @@ class ui(object):
         return choseninterface
 
     def interactive(self):
-        '''is interactive input allowed?
+        """is interactive input allowed?
 
         An interactive session is a session where input can be reasonably read
         from `sys.stdin'. If this function returns false, any attempt to read
@@ -1521,7 +1583,7 @@ class ui(object):
         to a terminal device.
 
         This function refers to input only; for output, see `ui.formatted()'.
-        '''
+        """
         i = self.configbool(b"ui", b"interactive")
         if i is None:
             # some environments replace stdin without implementing isatty
@@ -1531,8 +1593,7 @@ class ui(object):
         return i
 
     def termwidth(self):
-        '''how wide is the terminal in columns?
-        '''
+        """how wide is the terminal in columns?"""
         if b'COLUMNS' in encoding.environ:
             try:
                 return int(encoding.environ[b'COLUMNS'])
@@ -1541,7 +1602,7 @@ class ui(object):
         return scmutil.termsize(self)[0]
 
     def formatted(self):
-        '''should formatted output be used?
+        """should formatted output be used?
 
         It is often desirable to format the output to suite the output medium.
         Examples of this are truncating long lines or colorizing messages.
@@ -1556,7 +1617,7 @@ class ui(object):
 
         This function refers to output only; for input, see `ui.interactive()'.
         This function always returns false when in plain mode, see `ui.plain()'.
-        '''
+        """
         if self.plain():
             return False
 
@@ -1642,7 +1703,9 @@ class ui(object):
             if not r:
                 r = default
             if self.configbool(b'ui', b'promptecho'):
-                self._writemsg(self._fmsgout, r, b"\n", type=b'promptecho')
+                self._writemsg(
+                    self._fmsgout, r or b'', b"\n", type=b'promptecho'
+                )
             return r
         except EOFError:
             raise error.ResponseExpected()
@@ -1716,45 +1779,45 @@ class ui(object):
                         raise EOFError
                     return l.rstrip(b'\n')
                 else:
-                    return getpass.getpass('')
+                    return encoding.strtolocal(getpass.getpass(''))
         except EOFError:
             raise error.ResponseExpected()
 
     def status(self, *msg, **opts):
-        '''write status message to output (if ui.quiet is False)
+        """write status message to output (if ui.quiet is False)
 
         This adds an output label of "ui.status".
-        '''
+        """
         if not self.quiet:
             self._writemsg(self._fmsgout, type=b'status', *msg, **opts)
 
     def warn(self, *msg, **opts):
-        '''write warning message to output (stderr)
+        """write warning message to output (stderr)
 
         This adds an output label of "ui.warning".
-        '''
+        """
         self._writemsg(self._fmsgerr, type=b'warning', *msg, **opts)
 
     def error(self, *msg, **opts):
-        '''write error message to output (stderr)
+        """write error message to output (stderr)
 
         This adds an output label of "ui.error".
-        '''
+        """
         self._writemsg(self._fmsgerr, type=b'error', *msg, **opts)
 
     def note(self, *msg, **opts):
-        '''write note to output (if ui.verbose is True)
+        """write note to output (if ui.verbose is True)
 
         This adds an output label of "ui.note".
-        '''
+        """
         if self.verbose:
             self._writemsg(self._fmsgout, type=b'note', *msg, **opts)
 
     def debug(self, *msg, **opts):
-        '''write debug message to output (if ui.debugflag is True)
+        """write debug message to output (if ui.debugflag is True)
 
         This adds an output label of "ui.debug".
-        '''
+        """
         if self.debugflag:
             self._writemsg(self._fmsgout, type=b'debug', *msg, **opts)
             self.log(b'debug', b'%s', b''.join(msg))
@@ -1829,7 +1892,7 @@ class ui(object):
             self.system(
                 b"%s \"%s\"" % (editor, name),
                 environ=environ,
-                onerr=error.Abort,
+                onerr=error.CanceledError,
                 errprefix=_(b"edit failed"),
                 blockedtag=b'editor',
             )
@@ -1850,12 +1913,12 @@ class ui(object):
         errprefix=None,
         blockedtag=None,
     ):
-        '''execute shell command with appropriate output stream. command
+        """execute shell command with appropriate output stream. command
         output will be redirected if fout is not stdout.
 
         if command fails and onerr is None, return status, else raise onerr
         object as exception.
-        '''
+        """
         if blockedtag is None:
             # Long cmds tend to be because of an absolute path on cmd. Keep
             # the tail end instead
@@ -1868,7 +1931,7 @@ class ui(object):
             rc = self._runsystem(cmd, environ=environ, cwd=cwd, out=out)
         if rc and onerr:
             errmsg = b'%s %s' % (
-                os.path.basename(cmd.split(None, 1)[0]),
+                procutil.shellsplit(cmd)[0],
                 procutil.explainexit(rc),
             )
             if errprefix:
@@ -1882,9 +1945,9 @@ class ui(object):
         return procutil.system(cmd, environ=environ, cwd=cwd, out=out)
 
     def traceback(self, exc=None, force=False):
-        '''print exception traceback if traceback printing enabled or forced.
+        """print exception traceback if traceback printing enabled or forced.
         only to call in exception handler. returns true if traceback
-        printed.'''
+        printed."""
         if self.tracebackflag or force:
             if exc is None:
                 exc = sys.exc_info()
@@ -1945,30 +2008,6 @@ class ui(object):
         if self._progbar is not None and self._progbar.printed:
             self._progbar.clear()
 
-    def progress(self, topic, pos, item=b"", unit=b"", total=None):
-        '''show a progress message
-
-        By default a textual progress bar will be displayed if an operation
-        takes too long. 'topic' is the current operation, 'item' is a
-        non-numeric marker of the current position (i.e. the currently
-        in-process file), 'pos' is the current numeric position (i.e.
-        revision, bytes, etc.), unit is a corresponding unit label,
-        and total is the highest expected pos.
-
-        Multiple nested topics may be active at a time.
-
-        All topics should be marked closed by setting pos to None at
-        termination.
-        '''
-        self.deprecwarn(
-            b"use ui.makeprogress() instead of ui.progress()", b"5.1"
-        )
-        progress = self.makeprogress(topic, unit, total)
-        if pos is not None:
-            progress.update(pos, item=item)
-        else:
-            progress.complete()
-
     def makeprogress(self, topic, unit=b"", total=None):
         """Create a progress helper for the specified topic"""
         if getattr(self._fmsgerr, 'structured', False):
@@ -2010,7 +2049,7 @@ class ui(object):
         self._loggers[name] = logger
 
     def log(self, event, msgfmt, *msgargs, **opts):
-        '''hook for logging facility extensions
+        """hook for logging facility extensions
 
         event should be a readily-identifiable subsystem, which will
         allow filtering.
@@ -2019,7 +2058,7 @@ class ui(object):
         *msgargs are %-formatted into it.
 
         **opts currently has no defined meanings.
-        '''
+        """
         if not self._loggers:
             return
         activeloggers = [
@@ -2039,7 +2078,7 @@ class ui(object):
             self._loggers = registeredloggers
 
     def label(self, msg, label):
-        '''style msg based on supplied label
+        """style msg based on supplied label
 
         If some color mode is enabled, this will add the necessary control
         characters to apply such color. In addition, 'debug' color mode adds
@@ -2047,7 +2086,7 @@ class ui(object):
 
         ui.write(s, 'label') is equivalent to
         ui.write(ui.label(s, 'label')).
-        '''
+        """
         if self._colormode is not None:
             return color.colorlabel(self, msg, label)
         return msg
@@ -2126,191 +2165,21 @@ class ui(object):
             if (b'ui', b'quiet') in overrides:
                 self.fixconfig(section=b'ui')
 
+    def estimatememory(self):
+        """Provide an estimate for the available system memory in Bytes.
 
-class paths(dict):
-    """Represents a collection of paths and their configs.
-
-    Data is initially derived from ui instances and the config files they have
-    loaded.
-    """
-
-    def __init__(self, ui):
-        dict.__init__(self)
-
-        for name, loc in ui.configitems(b'paths', ignoresub=True):
-            # No location is the same as not existing.
-            if not loc:
-                continue
-            loc, sub = ui.configsuboptions(b'paths', name)
-            self[name] = path(ui, name, rawloc=loc, suboptions=sub)
-
-    def getpath(self, name, default=None):
-        """Return a ``path`` from a string, falling back to default.
-
-        ``name`` can be a named path or locations. Locations are filesystem
-        paths or URIs.
-
-        Returns None if ``name`` is not a registered path, a URI, or a local
-        path to a repo.
+        This can be overriden via ui.available-memory. It returns None, if
+        no estimate can be computed.
         """
-        # Only fall back to default if no path was requested.
-        if name is None:
-            if not default:
-                default = ()
-            elif not isinstance(default, (tuple, list)):
-                default = (default,)
-            for k in default:
-                try:
-                    return self[k]
-                except KeyError:
-                    continue
-            return None
-
-        # Most likely empty string.
-        # This may need to raise in the future.
-        if not name:
-            return None
-
-        try:
-            return self[name]
-        except KeyError:
-            # Try to resolve as a local path or URI.
+        value = self.config(b'ui', b'available-memory')
+        if value is not None:
             try:
-                # We don't pass sub-options in, so no need to pass ui instance.
-                return path(None, None, rawloc=name)
-            except ValueError:
-                raise error.RepoError(_(b'repository %s does not exist') % name)
-
-
-_pathsuboptions = {}
-
-
-def pathsuboption(option, attr):
-    """Decorator used to declare a path sub-option.
-
-    Arguments are the sub-option name and the attribute it should set on
-    ``path`` instances.
-
-    The decorated function will receive as arguments a ``ui`` instance,
-    ``path`` instance, and the string value of this option from the config.
-    The function should return the value that will be set on the ``path``
-    instance.
-
-    This decorator can be used to perform additional verification of
-    sub-options and to change the type of sub-options.
-    """
-
-    def register(func):
-        _pathsuboptions[option] = (attr, func)
-        return func
-
-    return register
-
-
-@pathsuboption(b'pushurl', b'pushloc')
-def pushurlpathoption(ui, path, value):
-    u = util.url(value)
-    # Actually require a URL.
-    if not u.scheme:
-        ui.warn(_(b'(paths.%s:pushurl not a URL; ignoring)\n') % path.name)
-        return None
-
-    # Don't support the #foo syntax in the push URL to declare branch to
-    # push.
-    if u.fragment:
-        ui.warn(
-            _(
-                b'("#fragment" in paths.%s:pushurl not supported; '
-                b'ignoring)\n'
-            )
-            % path.name
-        )
-        u.fragment = None
-
-    return bytes(u)
-
-
-@pathsuboption(b'pushrev', b'pushrev')
-def pushrevpathoption(ui, path, value):
-    return value
-
-
-class path(object):
-    """Represents an individual path and its configuration."""
-
-    def __init__(self, ui, name, rawloc=None, suboptions=None):
-        """Construct a path from its config options.
-
-        ``ui`` is the ``ui`` instance the path is coming from.
-        ``name`` is the symbolic name of the path.
-        ``rawloc`` is the raw location, as defined in the config.
-        ``pushloc`` is the raw locations pushes should be made to.
-
-        If ``name`` is not defined, we require that the location be a) a local
-        filesystem path with a .hg directory or b) a URL. If not,
-        ``ValueError`` is raised.
-        """
-        if not rawloc:
-            raise ValueError(b'rawloc must be defined')
-
-        # Locations may define branches via syntax <base>#<branch>.
-        u = util.url(rawloc)
-        branch = None
-        if u.fragment:
-            branch = u.fragment
-            u.fragment = None
-
-        self.url = u
-        self.branch = branch
-
-        self.name = name
-        self.rawloc = rawloc
-        self.loc = b'%s' % u
-
-        # When given a raw location but not a symbolic name, validate the
-        # location is valid.
-        if not name and not u.scheme and not self._isvalidlocalpath(self.loc):
-            raise ValueError(
-                b'location is not a URL or path to a local '
-                b'repo: %s' % rawloc
-            )
-
-        suboptions = suboptions or {}
-
-        # Now process the sub-options. If a sub-option is registered, its
-        # attribute will always be present. The value will be None if there
-        # was no valid sub-option.
-        for suboption, (attr, func) in pycompat.iteritems(_pathsuboptions):
-            if suboption not in suboptions:
-                setattr(self, attr, None)
-                continue
-
-            value = func(ui, self, suboptions[suboption])
-            setattr(self, attr, value)
-
-    def _isvalidlocalpath(self, path):
-        """Returns True if the given path is a potentially valid repository.
-        This is its own function so that extensions can change the definition of
-        'valid' in this case (like when pulling from a git repo into a hg
-        one)."""
-        try:
-            return os.path.isdir(os.path.join(path, b'.hg'))
-        # Python 2 may return TypeError. Python 3, ValueError.
-        except (TypeError, ValueError):
-            return False
-
-    @property
-    def suboptions(self):
-        """Return sub-options and their values for this path.
-
-        This is intended to be used for presentation purposes.
-        """
-        d = {}
-        for subopt, (attr, _func) in pycompat.iteritems(_pathsuboptions):
-            value = getattr(self, attr)
-            if value is not None:
-                d[subopt] = value
-        return d
+                return util.sizetoint(value)
+            except error.ParseError:
+                raise error.ConfigError(
+                    _(b"ui.available-memory value is invalid ('%s')") % value
+                )
+        return util._estimatememory()
 
 
 # we instantiate one globally shared progress bar to avoid
